//
//  AppDelegate.m
//  TestCocoaApp
//

#import "AppDelegate.h"
#import <AppKit/AppKitDefines.h>
#import <ApplicationServices/ApplicationServices.h>
#include <CoreGraphics/CoreGraphics.h>
#include <CoreFoundation/CoreFoundation.h>


@interface MyTestView : NSView  <CALayerDelegate>

#define USE_CUSTOM_LAYER 1

#if USE_CUSTOM_LAYER
@property CALayer *customLayer;
#endif

@end

@implementation MyTestView

- (id)initWithFrame:(NSRect)frame
{
	self = [super initWithFrame:frame];
	if (self) {
		self.wantsLayer = YES;
#if USE_CUSTOM_LAYER
        self.customLayer = [[CALayer alloc] init];
        self.customLayer.drawsAsynchronously = YES;
        [self.layer setDelegate:self];
        [self.layer addSublayer: self.customLayer];
#else
		self.layer.drawsAsynchronously = YES;
#endif
	}

	return self;
}


#if USE_CUSTOM_LAYER
- (void)drawLayer:(CALayer *)layer inContext:(CGContextRef)ctx {
    CFStringRef cfStr = CFStringCreateWithCString (0, "potistack_big.png", kCFStringEncodingUTF8);
    CFURLRef url = CFBundleCopyResourceURL (CFBundleGetMainBundle(), cfStr, NULL, NULL);
    
    NSSize imageSize;
    imageSize.width = 0;
    imageSize.height = 0;
    
    CGImageSourceRef imageSource = CGImageSourceCreateWithURL (url, 0);
    CFRetain (imageSource);
    CFDictionaryRef properties = CGImageSourceCopyPropertiesAtIndex (imageSource, 0, 0);
    CFNumberRef value = (CFNumberRef)CFDictionaryGetValue (properties, kCGImagePropertyPixelHeight);
    if (value)
    {
        double fValue = 0;
        if (CFNumberGetValue (value, kCFNumberDoubleType, &fValue))
            imageSize.height = fValue;
    }
    value = (CFNumberRef)CFDictionaryGetValue (properties, kCGImagePropertyPixelWidth);
    if (value)
    {
        double fValue = 0;
        if (CFNumberGetValue (value, kCFNumberDoubleType, &fValue))
            imageSize.width = fValue;
    }
    
    static CFStringRef kCGImageSourceShouldPreferRGB32 = CFSTR("kCGImageSourceShouldPreferRGB32");
    const void* keys[] = {kCGImageSourceShouldCache, kCGImageSourceShouldPreferRGB32};
    const void* values[] = {kCFBooleanTrue, kCFBooleanTrue};
    CFDictionaryRef options = CFDictionaryCreate (0, keys, values, 2, 0, 0);
    CGImageRef image = CGImageSourceCreateImageAtIndex (imageSource, 0, options);
    
    CGRect renderRect = CGRectMake(0., 0., imageSize.width, imageSize.height);
    
    CGContextRef context = ctx;
    // CGContextRef context = [[NSGraphicsContext currentContext] graphicsPort];

    float bitmapScaleFactor = 2;
    
    CGContextConcatCTM(context, CGAffineTransformMakeScale ((1. / (float) bitmapScaleFactor),
                                                              (1. / (float) bitmapScaleFactor)));
        
    CGContextDrawImage(context, renderRect, image);
}
#endif



- (void)drawRect:(NSRect)dirtyRect
{
#if !USE_CUSTOM_LAYER
    CFStringRef cfStr = CFStringCreateWithCString (0, "potistack_big.png", kCFStringEncodingUTF8);
    CFURLRef url = CFBundleCopyResourceURL (CFBundleGetMainBundle(), cfStr, NULL, NULL);

    NSSize imageSize;
    imageSize.width = 0;
    imageSize.height = 0;

    CGImageSourceRef imageSource = CGImageSourceCreateWithURL (url, 0);
    CFRetain (imageSource);
    CFDictionaryRef properties = CGImageSourceCopyPropertiesAtIndex (imageSource, 0, 0);
    CFNumberRef value = (CFNumberRef)CFDictionaryGetValue (properties, kCGImagePropertyPixelHeight);
    if (value)
    {
        double fValue = 0;
        if (CFNumberGetValue (value, kCFNumberDoubleType, &fValue))
            imageSize.height = fValue;
    }
    value = (CFNumberRef)CFDictionaryGetValue (properties, kCGImagePropertyPixelWidth);
    if (value)
    {
        double fValue = 0;
        if (CFNumberGetValue (value, kCFNumberDoubleType, &fValue))
            imageSize.width = fValue;
    }

    static CFStringRef kCGImageSourceShouldPreferRGB32 = CFSTR("kCGImageSourceShouldPreferRGB32");
    const void* keys[] = {kCGImageSourceShouldCache, kCGImageSourceShouldPreferRGB32};
    const void* values[] = {kCFBooleanTrue, kCFBooleanTrue};
    CFDictionaryRef options = CFDictionaryCreate (0, keys, values, 2, 0, 0);
    CGImageRef image = CGImageSourceCreateImageAtIndex (imageSource, 0, options);

    CGRect renderRect = CGRectMake(0., 0., imageSize.width, imageSize.height);

    CGContextRef context = [[NSGraphicsContext currentContext] graphicsPort];

    float bitmapScaleFactor = 2;

    CGContextConcatCTM(context, CGAffineTransformMakeScale ((1. / (float) bitmapScaleFactor),
                                                            (1. / (float) bitmapScaleFactor)));

    CGContextDrawImage(context, renderRect, image);
#endif

}



@end



@interface AppDelegate ()

@property (weak) IBOutlet NSWindow *window;
@end

@implementation AppDelegate

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)sender {
    return YES;
}


- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
   
   NSRect frame = NSMakeRect(0, 0, 300, 300);
    
   NSView *superview = [_window contentView];
	
	MyTestView *testView = [[MyTestView alloc] initWithFrame:frame];
	[superview addSubview:testView];
}

@end
